/**
 * @author  Petros Melissaridis
 * @version 1.0
 * @see     java.awt.BaseWindow
 * @see     java.awt.Button
 */
package org.supermarket.gr;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

/**
 *  Η κλάση αυτή είναι υπεύθυνη για το σχεδιασμό του UI για το login του χρήστη
 *
 */
public class LoginUserUI extends Activity implements OnClickListener
{
	protected EditText username;
	protected EditText password;
	protected Button loginButton;
	protected Button accountButton;	
	/**
     * Κλειδί αποθηκευμένο στα SharedPreferences που αφορά την το ID του χρήστη που είναι συνδεδεμένος στην εφαρμογή
    */
 	public static final String USER_STORED = "User_stored";
 	/**
 	 *  Δισδιάτος πίνακας που αποθηκεύει τα δεδομένα της τοπικής βάσης δεδομένων που δεν έχουν σταλθεί στη κεντρική βάση δεδομένων. 
 	 *
 	 */
 	protected String[][] incompleteRequests;
 	
 	/**
 	 *  Κλειδί καταχωρημένο στα SharedPreferences που αφορά τις προυποθέσεις για εκτέλεση ενεργειών ενημέρωσεις 
 	 *
 	 */
	public static final String PREFS_NAME = "Update_settings";
	
	/**
     * Συνάρτηση στην οποία γίνεται ο σχεδιασμός του UI
    */
	@Override
	protected void onCreate(Bundle savedInstanceState) 
	{
		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);
		setContentView(R.layout.login);
		
		setActions();
	}
	
	/**
	 *  Ορισμός των συστατικών του UI 
	 *
	 */
	void setActions()
	{	
		loginButton=(Button) findViewById(R.id.login_button);
		loginButton.setOnClickListener(this);
		accountButton=(Button) findViewById(R.id.account_button);
		accountButton.setOnClickListener(this);
        username=(EditText) findViewById(R.id.username);
        password=(EditText) findViewById(R.id.password);
	}
	
	
	
	/**
     * 
     * Συναρτηση που αναζητά για καταχωρήσεις στη τοπική βάση δεδομένων οι οποίες δεν έχουν σταλθεί στο server
    */
	protected void updateTasks()
    {
    	DatabaseCommunication ed;
 	    ed=new DatabaseCommunication(this);
     	Cursor cursor;
       
        cursor=ed.retrieveAllNoStoredProductsLocally();
        
       if(cursor.getCount()>0)
		{
    	   incompleteRequests=new String[cursor.getCount()][9];
    	   int counter=0;
	   		while(cursor.moveToNext())
	   		{
	   			incompleteRequests[counter][0]=new String(cursor.getString(cursor.getColumnIndex("pname")));
	   			incompleteRequests[counter][1]=new String(cursor.getString(cursor.getColumnIndex("barcode")));
	   			incompleteRequests[counter][2]=new String(cursor.getString(cursor.getColumnIndex("super_name")));
	   			incompleteRequests[counter][3]=new String(cursor.getString(cursor.getColumnIndex("amount")));
	
	   			incompleteRequests[counter][4]=new String(cursor.getString(cursor.getColumnIndex("units")));
	   			incompleteRequests[counter][5]=new String(cursor.getString(cursor.getColumnIndex("is_offer")));
	
	   			incompleteRequests[counter][6]=new String(cursor.getString(cursor.getColumnIndex("is_packed")));
	   			incompleteRequests[counter][7]=new String(cursor.getString(cursor.getColumnIndex("is_packed_items")));
	   			incompleteRequests[counter][8]=new String(cursor.getString(cursor.getColumnIndex("value")));
	   			counter++;
	   		}
	   		cursor.close();
	   		ed.closeDB();
	   		LoginUserSendLocalData task = new LoginUserSendLocalData(this,incompleteRequests);
    	   task.execute(new String[] { "" });
		}
       else
       {
    	   cursor.close();
	   		ed.closeDB();
    	   Intent firstActivity = new Intent(getApplicationContext(),MainActivityUI.class);
           startActivity(firstActivity);
	    	finish();
       }
		 
      
    }
	
	/**
     * Συναρτηση για τον έλεγχο συνδεσιμότητας με το Internet.
    */
	 public boolean isOnline() 
	 {
		 boolean connected = false;
	
		ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo ni = cm.getActiveNetworkInfo();
			
		if (ni!=null && ni.isAvailable() && ni.isConnected()) 
		{
			connected=true;
		} 
		else 
		{
			connected=false; 
		}
		return connected;
	}	
    
	 /**
	  * Συναρτηση για τον έλεγχο του τύπου σύνδεσης (3G,WiFi)
	  */
    public boolean isFastConnection() 
	 { 
		 ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		 NetworkInfo ni = cm.getActiveNetworkInfo();
			
			if(ni.getType()==ConnectivityManager.TYPE_MOBILE)
				return false;
			else 	
				return true;
	 }
	
    
    
    /**
	 *  Handle two different events. The first one is because the login button was tapped and the second one is
	 *   because the account button was tapped 
	 *
	 *	In the case the login button was tapped, the local database is checked for a record with username and password 
	 *	equal with the ones user edit at edit fields.
	 *	If does not exist a record a query is sent to database server in case there is an internet connection.
	 *	If the record exists at server then the record is stored to local database and the session starts and 
	 *	is stored to share preferences.
	 *	If it is not possible to connect to server then the user cannot go to main menu of application. 
	 *
	 *	At the case the account button was tapped the account activity  (class) is called
	 */
	public void onClick(View v) 
	{
		 switch(v.getId())
	      {
			     case R.id.login_button:
			    	 Pattern pattern = Pattern.compile("\\s");
			         Matcher matcherUsernameLogin;
			         Matcher matcherPasswordLogin;
			         
			         matcherUsernameLogin = pattern.matcher((CharSequence) username.getText().toString());
			         matcherPasswordLogin = pattern.matcher((CharSequence) password.getText().toString());
			    	 
			    	 if(!matcherUsernameLogin.find() && !matcherPasswordLogin.find())
			    	 {
			    	 		String theUsernameLogin=username.getText().toString();
			    	 		String thePasswordLogin=password.getText().toString();
			    		 if(theUsernameLogin!=null && theUsernameLogin.length()>0 &&  thePasswordLogin!=null && thePasswordLogin.length()>0)
				    	 {
			    			 DatabaseCommunication ed=new DatabaseCommunication(this);
			    			 Cursor cursor=ed.retrieveUserLogin(theUsernameLogin, thePasswordLogin);
				    		 
				    		 if(cursor.getCount()==0)
				    		 {
				    			 cursor.close();
				    			 ed.closeDB();
				    			 LoginUserCommunicate task = new LoginUserCommunicate(this,theUsernameLogin,thePasswordLogin);
					    	     task.execute(new String[] { "" });
				    		 }
				    		 //We find match of registration in the local database
				    		 else
				    		 {
				    			 	while(cursor.moveToNext())
							 		{
							 			SharedPreferences user = getSharedPreferences(USER_STORED, 0);
							 		    SharedPreferences.Editor editor = user.edit();
							 			editor.putString("username", cursor.getString(cursor.getColumnIndexOrThrow("username")));
							 			editor.putString("password",cursor.getString(cursor.getColumnIndexOrThrow("password")));
							 			editor.putString("user_id",cursor.getString(cursor.getColumnIndexOrThrow("user_id")));
							 			editor.commit();
							 			SupermarketsAndPricesStaticClass.setUserID(cursor.getString(cursor.getColumnIndexOrThrow("user_id")));
							 		}
				    			 
				    			 	cursor.close();
								 	ed.closeDB();
				    			 
								 	if(isOnline()==true)
				    	      	  	{
				    	    			boolean updateDatabase=false;
				    	    	    	SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);
				    	    	    	String updateSetting = settings.getString("update", "always");
				    	    	    	if(updateSetting.equals("always"))
				    	    	    		updateDatabase=true;
				    	    	    	else if(updateSetting.equals("wifi"))
				    	    	    	{
				    	    	    		if(isFastConnection())
				    	    	    			updateDatabase=true;
				    	    	    	}
				    	    	    	
				    	    	    	if(updateDatabase)
				    	    	    		updateTasks();
				    	    	    	else
				    	    	    	{
				    	    	    		Intent firstActivity = new Intent(getApplicationContext(),MainActivityUI.class);
				    	                    startActivity(firstActivity);
				    	        	    	finish();
				    	    	    	}
				    	      	  	}
				    	    		else
				    	    		{
				    	    			Intent firstActivity = new Intent(getApplicationContext(),MainActivityUI.class);
				    	                startActivity(firstActivity);
				    	    	    	finish();
				    	    		}
				    		 }
				    	 }
			    		 else
			    			 Toast.makeText(this,R.string.field_not_edit,Toast.LENGTH_LONG).show();
			    	 }
			    	 else
			    		 Toast.makeText(this,R.string.no_white_characters,Toast.LENGTH_LONG).show();
			 		break;
			     case R.id.account_button:
			    	Intent newAccount = new Intent(getApplicationContext(),CreateUserAccountUI.class);
 	                startActivity(newAccount);
	      }
	}
}	
	