/**
 * @author  Petros Melissaridis
 * @version 1.0
 * @see     java.awt.BaseWindow
 * @see     java.awt.Button
 */
package org.supermarket.gr;


import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.widget.ProgressBar;

/*
 * First of all at this class I put on the screen the splash screen. Then I check if there is already a active session. 
 * An active session exists in case the user had been connected in application before, without logout later. 
 * If there is an active session, I check for connectivity with Internet (3G or wIFi) for continue with update tasks. 
 * If there is not connectivity or the user is connected via 3G and have specified that only in case he is connected 
 * with wifi updates should be done, then I keep the splash screen for a while and continue with FirstActivity class.
 * In case there is the connectivity the user prefer for do the update task, I start the update process.
 */

/**
 *Η SuperMarketActivity είναι η πρώτη κλάση που καλείται κατά την είσοδο στην εφαρμογή. Σε αυτήν λαμβάνουν χώρα διάφορες ενέργειες όπως είναι η αποστολή 
 *των δεδομένων των χρηστών στη κεντρική βάση και η αναζήτηση για ενεργό session  χρήστη. 
 */
public class SuperMarketActivity extends Activity 
{
	/*
     * Use to have access to resources like layout attributes  
    */
	
	/**
     * Κλειδί καταχωρημένο στα SharedPreferences που αφορά την ύπαρξη ενεργού session
    */
	public static final String USER_STORED = "User_stored";
	/**
     * Κλειδί καταχωρημένο στα SharedPreferences που αφορά τις προυποθέσεις για εκτέλεση εναεργειών ενημέρωσεις
    */
	public static final String PREFS_NAME = "Update_settings";
	/*
     * Array where the data, which have not stored yet in server database, are stored. Each row is equivalent with a row in table
     * with the elements which have not stored yet in server database.
    */
	/**
     * Δισδιάτος πίνακας που αποθηκεύει τα δεδομένα της τοπικής βάσης δεδομένων που δεν έχουν σταλθεί στη κεντρική βάση δεδομένων.
    */
	protected String[][] incompleteRequests;
	
	
	/**
     * Μεταβλητή που αποθηκεύει το όνομα της κλάσης στην οποία θα οδηγηθεί η εφαρμογή μετα την ολοκλήρωση της εκτέλεσης της SuperMarketActivity
    */
	public String whereToGoAfterSplash;
	
	/**
     * Progress bar
    */
	private ProgressBar mProgress;
	/**
     * Progress status
    */
	private int mProgressStatus = 0;
	
	/**
     * Συνάρτηση στην οποία γίνεται ο σχεδιασμός του UI
    */
    @Override
    public void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.splash);
        
        mProgress = (ProgressBar) findViewById(R.id.progress_bar);
        mProgress.setProgress(mProgressStatus);
        
        //elegxw an yparxei kapoio apothikeymeno session apo prohgoumeno login
        SharedPreferences session = getSharedPreferences(USER_STORED, 0);
    	String getSession = session.getString("username", "nill");
    	String theUserID = session.getString("user_id", "nill");
    	
    	//an den yparxei session tote eite tha kanei login o xristis h tha dhmioyrghsei neo logarismo
    	if(getSession.equals("nill"))
        {
    		whereToGoAfterSplash=new String("login");
    		additionalDelay();
        }
    	else
    	{
    		SupermarketsAndPricesStaticClass.setUserID(theUserID);
    		if(isOnline()==true)
      	  	{
    			boolean updateDatabase=false;
    	    	SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);
    	    	String updateSetting = settings.getString("update", "always");
    	    	if(updateSetting.equals("always"))
    	    		updateDatabase=true;
    	    	else if(updateSetting.equals("wifi"))
    	    	{
    	    		if(isFastConnection())
    	    			updateDatabase=true;
    	    	}
    	    	
    	    	if(updateDatabase)
    	    		updateTasks();
    	    	else
    	    	{
    	    		whereToGoAfterSplash=new String("firstActivity");
        			additionalDelay();
    	    	}
      	  	}
    		else
    		{
    			whereToGoAfterSplash=new String("firstActivity");
    			additionalDelay();
    		}
    	}
    	
    }
    
   
    /**
     * Συναρτηση που διατηρεί για λίγα δευτερόπλεπτα την splash screen στη περίπτωση που δεν υπάρχει εργασία ανανέωσης
    */
    protected void additionalDelay()
	 {
		 	final int welcomeScreenDisplay = 3000;
		  
		    Thread welcomeThread = new Thread() 
		    {
		    	int wait = 0;

		    	@Override
		    	public void run() 
		    	{
		    		try 
		    		{
		    			super.run();
		    			
		    			while (wait < welcomeScreenDisplay)
		    			{
		    				sleep(100);
		    				wait += 100;
		    			}
		    		} 
		    		catch (Exception e) 
		    		{
		    			
		    		} 
		    		finally 
		    		{
		    			if(whereToGoAfterSplash.equals("login"))
		    			{
		    				Intent loginUser = new Intent(getApplicationContext(), LoginUserUI.class);
			                startActivity(loginUser);
			                finish();
		    			}
		    			else
		    			{
		    				
		    				Intent firstActivity = new Intent(getApplicationContext(),MainActivityUI.class);
		                    startActivity(firstActivity);
		        	    	finish();
		    			}
		    		}
		    	}
		    };
		    welcomeThread.start();
	 }
    
    /**
     * Συναρτηση για τον έλεγχο συνδεσιμότητας με το Internet.
     * 
    */
    public boolean isOnline() 
	 {
		 boolean connected = false;
	
		ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo ni = cm.getActiveNetworkInfo();
			
		if (ni!=null && ni.isAvailable() && ni.isConnected()) 
		{
			connected=true;
		} 
		else 
		{
			connected=false; 
		}
		return connected;
	}
    
    /**
     * Συναρτηση για τον έλεγχο του τύπου σύνδεσης (3G,WiFi)
    */
    public boolean isFastConnection() 
	 { 
		 ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		 NetworkInfo ni = cm.getActiveNetworkInfo();
			
			if(ni.getType()==ConnectivityManager.TYPE_MOBILE)
				return false;
			else 	
				return true;
	 }
    
    /**
     * Συναρτηση που αναζητά για καταχωρήσεις στη τοπική βάση δεδομένων οι οποίες δεν έχουν σταλθεί στο server
    */
    protected void updateTasks()
    {
    	DatabaseCommunication ed;
 	    ed=new DatabaseCommunication(this);
     	Cursor cursor;
        
        
        cursor=ed.retrieveAllNoStoredProductsLocally();
        
       if(cursor.getCount()>0)
		{
    	   incompleteRequests=new String[cursor.getCount()][9];
    	   int counter=0;
	   		while(cursor.moveToNext())
	   		{
	   			incompleteRequests[counter][0]=new String(cursor.getString(cursor.getColumnIndex("pname")));
	   			incompleteRequests[counter][1]=new String(cursor.getString(cursor.getColumnIndex("barcode")));
	   			incompleteRequests[counter][2]=new String(cursor.getString(cursor.getColumnIndex("super_name")));
	   			incompleteRequests[counter][3]=new String(cursor.getString(cursor.getColumnIndex("amount")));
	
	   			incompleteRequests[counter][4]=new String(cursor.getString(cursor.getColumnIndex("units")));
	   			incompleteRequests[counter][5]=new String(cursor.getString(cursor.getColumnIndex("is_offer")));
	
	   			incompleteRequests[counter][6]=new String(cursor.getString(cursor.getColumnIndex("is_packed")));
	   			incompleteRequests[counter][7]=new String(cursor.getString(cursor.getColumnIndex("is_packed_items")));
	   			incompleteRequests[counter][8]=new String(cursor.getString(cursor.getColumnIndex("value")));
	   			counter++;
	   		}
	   		cursor.close();
	   		ed.closeDB();
	   		SendUpdatesToServerDatabase task = new SendUpdatesToServerDatabase(this);
    	   task.execute(new String[] { "" });
		}
       else
       {
    	   cursor.close();
	   		ed.closeDB();
	   		whereToGoAfterSplash=new String("firstActivity");
			additionalDelay();
       }
		 
      
    }
    
}	
  